﻿#region License Information
/* HeuristicLab
 * Copyright (C) 2002-2012 Heuristic and Evolutionary Algorithms Laboratory (HEAL)
 *
 * This file is part of HeuristicLab.
 *
 * HeuristicLab is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * HeuristicLab is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with HeuristicLab. If not, see <http://www.gnu.org/licenses/>.
 */
#endregion

using System;
using System.Collections.Generic;
using System.Linq;
using HeuristicLab.Common;
using HeuristicLab.Core;
using HeuristicLab.Data;
using HeuristicLab.Operators;
using HeuristicLab.Optimization;
using HeuristicLab.Parameters;
using HeuristicLab.Persistence.Default.CompositeSerializers.Storable;
using HeuristicLab.Random;

namespace HeuristicLab.Encodings.DecisionList {
  [Item("DecisionListEvaluator", "Description missing")]
  [StorableClass]
  public abstract class DecisionListEvaluator : SingleSuccessorOperator, IDecisionListEvaluator, IDecisionListOperator, IStochasticOperator {

    #region Parameter Properties
    public ILookupParameter<IRandom> RandomParameter {
      get { return (ILookupParameter<IRandom>)Parameters["Random"]; }
    }
    public ILookupParameter<DecisionList> DecisionListParameter {
      get { return (ILookupParameter<DecisionList>)Parameters["DecisionList"]; }
    }
    public IValueLookupParameter<IntRange> EvaluationPartitionParameter {
      get { return (IValueLookupParameter<IntRange>)Parameters["EvaluationPartition"]; }
    }
    public IValueLookupParameter<PercentValue> RelativeNumberOfEvaluatedSamplesParameter {
      get { return (IValueLookupParameter<PercentValue>)Parameters["RelativeNumberOfEvaluatedSamples"]; }
    }
    public IValueLookupParameter<IDecisionListClassificationProblemData> ProblemDataParameter {
      get { return (IValueLookupParameter<IDecisionListClassificationProblemData>)Parameters["ProblemData"]; }
    }
    public IValueLookupParameter<IntValue> SizePenaltyMinRulesParameter {
      get { return (IValueLookupParameter<IntValue>)Parameters["SizePenaltyMinRules"]; }
    }
    public ILookupParameter<DoubleValue> QualityParameter {
      get { return (ILookupParameter<DoubleValue>)Parameters["Quality"]; }
    }
    #endregion

    [StorableConstructor]
    protected DecisionListEvaluator(bool deserializing) : base(deserializing) { }
    protected DecisionListEvaluator(DecisionListEvaluator original, Cloner cloner)
      : base(original, cloner) {
    }
    public DecisionListEvaluator()
      : base() {
      Parameters.Add(new LookupParameter<IRandom>("Random", "The random generator to use."));
      Parameters.Add(new LookupParameter<DecisionList>("DecisionList", ""));
      Parameters.Add(new ValueLookupParameter<IntRange>("EvaluationPartition", ""));
      Parameters.Add(new ValueLookupParameter<PercentValue>("RelativeNumberOfEvaluatedSamples", ""));
      Parameters.Add(new ValueLookupParameter<IDecisionListClassificationProblemData>("ProblemData", ""));
      Parameters.Add(new ValueLookupParameter<IntValue>("SizePenaltyMinRules", ""));
      Parameters.Add(new LookupParameter<DoubleValue>("Quality", ""));
    }

    protected IEnumerable<int> GenerateRowsToEvaluate() {
      return GenerateRowsToEvaluate(RelativeNumberOfEvaluatedSamplesParameter.ActualValue.Value);
    }

    protected IEnumerable<int> GenerateRowsToEvaluate(double percentageOfRows) {
      IEnumerable<int> rows;
      int samplesStart = EvaluationPartitionParameter.ActualValue.Start;
      int samplesEnd = EvaluationPartitionParameter.ActualValue.End;
      int testPartitionStart = ProblemDataParameter.ActualValue.TestPartition.Start;
      int testPartitionEnd = ProblemDataParameter.ActualValue.TestPartition.End;
      if (samplesEnd < samplesStart) throw new ArgumentException("Start value is larger than end value.");

      if (percentageOfRows.IsAlmost(1.0))
        rows = Enumerable.Range(samplesStart, samplesEnd - samplesStart);
      else {
        int seed = RandomParameter.ActualValue.Next();
        int count = (int)((samplesEnd - samplesStart) * percentageOfRows);
        if (count == 0) count = 1;
        rows = RandomEnumerable.SampleRandomNumbers(seed, samplesStart, samplesEnd, count);
      }

      rows = rows.Where(i => i < testPartitionStart || testPartitionEnd <= i);
      return rows;
    }
  }
}
